@@ -8,23 +8,24 @@ use warnings FATAL => 'all';
 
 =head1 NAME
 
-Run::Parts - Perl interface to Debian's run-parts tool
+Run::Parts - Offers functionality of Debian's run-parts tool in Perl.
 
 =head1 VERSION
 
-Version 0.01
+Version 0.03
 
 =cut
 
-our $VERSION = '0.02';
+our $VERSION = '0.03';
 
 
 =head1 SYNOPSIS
 
-Perl interface to Debian's run-parts tool. run-parts runs all the
-executable files named within constraints described below, found in
-the given directory.  Other files and directories are silently
-ignored.
+Run::Parts offers functionality of Debian's run-parts tool in Perl.
+
+run-parts runs all the executable files named within constraints
+described below, found in the given directory.  Other files and
+directories are silently ignored.
 
 Additionally it can just print the names of the all matching files
 (not limited to executables, but ignores blacklisted files like
@@ -37,17 +38,27 @@ Perhaps a little code snippet.
 
     use Run::Parts;
 
-    my $rp = Run::Parts->new('directory);
+    my $rp  = Run::Parts->new('directory'); # chooses backend automatically
+    my $rpp = Run::Parts->new('directory', 'perl'); # pure perl backend
+    my $rpd = Run::Parts->new('directory', 'debian'); # uses /bin/run-parts
 
     my @file_list        = $rp->list;
-    my @executables_list = $rp->test;
-    my $commands_output  = $rp->run;
-    …
+    my @executables_list = $rpp->test;
+    my $commands_output  = $rpd->run;
+    ...
 
-=head1 EXPORT
+=head1 BACKENDS
 
-A list of functions that can be exported.  You can delete this section
-if you don't export anything, such as for a purely object-oriented module.
+Run::Parts contains two backend implementation. Run::Parts::Debian
+actually uses /bin/run-parts and Run::Parts::Perl is a pure Perl
+implementation of a basic set of run-parts' functionality.
+
+Run::Parts::Debian may or may not work with RedHat's simplified
+shell-script based reimplementation of Debian's run-parts.
+
+By default Run::Parts uses Run::Parts::Debian if /bin/run-parts
+exists, Run::Parts::Perl otherwise. But you can also choose any of the
+backends explicitly.
 
 =head1 METHODS
 
@@ -63,7 +74,30 @@ sub new {
     bless($self, shift);
     $self->{dir} = shift;
 
-    return $self
+    my $backend = shift;
+    if (defined $backend) {
+        if (ref $backend) {
+            $self->{backend} = $backend->new($self->{dir});
+        } elsif ($backend eq 'debian' or $backend eq 'run-parts') {
+            use Run::Parts::Debian;
+            $self->{backend} = Run::Parts::Debian->new($self->{dir});
+        } elsif ($backend eq 'perl' or $backend eq 'module') {
+            use Run::Parts::Perl;
+            $self->{backend} = Run::Parts::Perl->new($self->{dir});
+        } else {
+            warn "Unknown backend $backend in use";
+            require $backend;
+            $self->{backend} = $backend->new($self->{dir});
+        }
+    } else {
+        if (-x '/bin/run-parts') {
+            $self->{backend} = Run::Parts::Debian->new($self->{dir});
+        } else {
+            $self->{backend} = Run::Parts::Perl->new($self->{dir});
+        }
+    }
+
+    return $self;
 }
 
 =head2 run_parts_command
@@ -74,16 +108,7 @@ Returns the run-parts to run with the given command parameter
 
 sub run_parts_command {
     my $self = shift;
-    my $rp_cmd = shift;
-
-    my $command =
-        "/bin/run-parts " .
-        ((defined($rp_cmd) and $rp_cmd ne '') ? "'--$rp_cmd'" : '') .
-        " '".$self->{dir}."'";
-
-    return wantarray ?
-        do { my @l = `$command`; chomp(@l); return @l } :
-        `$command`;
+    return $self->{backend}->run_parts_command(@_);
 }
 
 =head2 list
@@ -124,7 +149,7 @@ sub run {
 
 =head1 SEE ALSO
 
-run-parts(8)
+run-parts(8), Run::Parts::Debian, Run::Parts::Perl
 
 =head1 AUTHOR
 
